/*
Copyright (c) 2002 Xavier Trochu

This software is provided 'as-is', without any express or implied warranty. In no event
will the authors be held liable for any damages arising from the use of this software.

Permission is granted to anyone to use this software for any purpose, including commercial
applications, and to alter it and redistribute it freely, subject to the following
restrictions:

1. The origin of this software must not be misrepresented; you must not claim that you
   wrote the original software. If you use this software in a product, an acknowledgment
   in the product documentation would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must not be misrepresented 
   as being the original software.

3. This notice may not be removed or altered from any source distribution.
*/
#include <stdexcept>
#include "brenderer.h"

using namespace std;

void base_renderer::clean() throw() {
	if(_data) {
		delete[] _data;
		_data = 0;
		_width = _height = 0;
	}
}

base_renderer::base_renderer() throw() : 
		_data(0), 
		_frame(0), 
		_nbframes(0), 
		_width(0), 
		_height(0) {
}

base_renderer::~base_renderer() throw() { 
	clean(); 
}

bool base_renderer::init_frame(const point & p) throw(exception) {
	clean();
	_width = p.get_x();
	_height = p.get_y();
	assert(_width && _height);
	_data = new char[_width * _height];
	if(!_data) throw runtime_error("base_renderer unable to allocate frame buffer");
	return true;
}

char * base_renderer::lock_frame(int frame) throw(exception) {
	_frame = frame; 
	if(!_data) throw runtime_error("no allocated image buffer in lock_frame");
	return _data;
}

bool base_renderer::unlock_frame() throw() {
	return true;
}

bool base_renderer::flip_frame() throw() { 
	save(); 
	return true;
}

bool base_renderer::set_palette(const palette & pal) throw() { 
	_pal = pal; 
	return true;
}
