/*
Copyright (c) 2002 Xavier Trochu

This software is provided 'as-is', without any express or implied warranty. In no event
will the authors be held liable for any damages arising from the use of this software.

Permission is granted to anyone to use this software for any purpose, including commercial
applications, and to alter it and redistribute it freely, subject to the following
restrictions:

1. The origin of this software must not be misrepresented; you must not claim that you
   wrote the original software. If you use this software in a product, an acknowledgment
   in the product documentation would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must not be misrepresented 
   as being the original software.

3. This notice may not be removed or altered from any source distribution.
*/
#ifndef __CODEC37_H_
#define __CODEC37_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#ifdef DEBUG
# ifndef NO_DEBUG_CODEC37
#  define DEBUG_CODEC37
# endif
#else
# ifdef DEBUG_CODEC37
#  error DEBUG_CODEC37 defined without DEBUG
# endif
#endif

#ifdef DEBUG_CODEC37
# ifndef NO_DEBUG_CODEC37_PROCS
#  define DEBUG_CODEC37_PROC1
#  define DEBUG_CODEC37_PROC2
#  define DEBUG_CODEC37_PROC3
#  define DEBUG_CODEC37_PROC4
# endif
#endif

#include <exception>
#include "decoder.h"

/*!	@brief ::decoder for codec 37.

*/
class codec37_decoder : public decoder {
private:
	int _deltaSize;
	unsigned char * _deltaBufs[2];
	unsigned char * _deltaBuf;
	short * _offset_table;
	int _curtable;
	unsigned short _prev_seq_nb;
	int _table_last_pitch;
	int _table_last_index;
	static const int DELTA_ADD = 0x3E00; // what is this 0x3E00 ?? == 320*200/4 - 128 
							// It looks like it is a safe-guarding protection from bugs., but maybe not...
public:
	bool init_size(const point &, const rect &) throw(std::exception);
	codec37_decoder() throw();
	void clean() throw();
	virtual ~codec37_decoder() throw();
protected:
	static inline unsigned int expand(unsigned char b) {
		unsigned int r = b | (b << 8);
		return r | (r << 16);
	}
	void maketable(int, int);
	void proc1(blitter &, chunck &, int, int, int, int);
	void proc2(blitter &, chunck &, int);
	void proc3_with_fdfe(blitter &, chunck &, int, int, int);
	void proc3_without_fdfe(blitter &, chunck &, int, int, int);
	void proc4(blitter &, chunck &, int, int, int);
public:
	bool decode(blitter &, chunck &) throw(std::exception);
};

#endif
