/*
Copyright (c) 2002 Xavier Trochu

This software is provided 'as-is', without any express or implied warranty. In no event
will the authors be held liable for any damages arising from the use of this software.

Permission is granted to anyone to use this software for any purpose, including commercial
applications, and to alter it and redistribute it freely, subject to the following
restrictions:

1. The origin of this software must not be misrepresented; you must not claim that you
   wrote the original software. If you use this software in a product, an acknowledgment
   in the product documentation would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must not be misrepresented 
   as being the original software.

3. This notice may not be removed or altered from any source distribution.
*/
#include <algorithm>
#include "color.h"

using namespace std;

color::color() throw() : _r(0), _g(0), _b(0) {
}

color::color(value_type r, value_type g, value_type b) throw() : _r(r), _g(g), _b(b) {
}

color::color(const color & c) throw() : _r(c._r), _g(c._g), _b(c._b) {
}

color::color & color::operator=(const color & c) {
	_r = c._r;
	_g = c._g;
	_b = c._b;
	return *this;
}

color::~color() throw() {
}

color::value_type color::red() const throw() { 
	return _r; 
}

color::value_type color::green() const throw() { 
	return _g; 
}

color::value_type color::blue() const throw() { 
	return _b;
}

void color::delta(short * ptr) throw() {
	// This is a very specific method for XPALs.
	int t;
	t = ((static_cast<int>(_r) << 7) + _r + ptr[0]) >> 7;
	_r = max(min(255, t), 0);
	t = ((static_cast<int>(_g) << 7) + _g + ptr[1]) >> 7;
	_g = max(min(255, t), 0);
	t = ((static_cast<int>(_b) << 7) + _b + ptr[2]) >> 7;
	_b = max(min(255, t), 0);
}

ostream & operator <<(ostream & o, const color & c) {
	o << "COLOR(" << static_cast<int>(c.red()) << ", " << static_cast<int>(c.green()) << ", " << static_cast<int>(c.blue()) << ")";
	return o;
}

