/*
Copyright (c) 2002 Xavier Trochu

This software is provided 'as-is', without any express or implied warranty. In no event
will the authors be held liable for any damages arising from the use of this software.

Permission is granted to anyone to use this software for any purpose, including commercial
applications, and to alter it and redistribute it freely, subject to the following
restrictions:

1. The origin of this software must not be misrepresented; you must not claim that you
   wrote the original software. If you use this software in a product, an acknowledgment
   in the product documentation would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must not be misrepresented 
   as being the original software.

3. This notice may not be removed or altered from any source distribution.
*/
#ifndef __COLOR_H_
#define __COLOR_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <ostream>

/*! 	@brief simple class for handling a color.

	This small class is an helper for colors.
*/
class color {
public:
	typedef unsigned char value_type;	//!< The type of the color components.
private:
	value_type _r;	//!< The red component.
	value_type _g;	//!< The green component.
	value_type _b;	//!< The blue component.
public:
	color() throw();
	color(value_type, value_type, value_type) throw();
	color(const color &) throw();
	color & operator=(const color &);
	virtual ~color() throw();
	value_type red() const throw();
	value_type green() const throw();
	value_type blue() const throw();
	/*!	@brief handle delta palette modification

		This method is used specifically by player::handleDeltaPalette().
		It updates the color component using delta values given as short.

		@param ptr pointer to a table of 3 shorts that contain delta values to use.
	*/
	void delta(short * ptr) throw();
};

extern std::ostream & operator <<(std::ostream &, const color &);

#endif
