/*
Copyright (c) 2002 Xavier Trochu

This software is provided 'as-is', without any express or implied warranty. In no event
will the authors be held liable for any damages arising from the use of this software.

Permission is granted to anyone to use this software for any purpose, including commercial
applications, and to alter it and redistribute it freely, subject to the following
restrictions:

1. The origin of this software must not be misrepresented; you must not claim that you
   wrote the original software. If you use this software in a product, an acknowledgment
   in the product documentation would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must not be misrepresented 
   as being the original software.

3. This notice may not be removed or altered from any source distribution.
*/
#ifndef __DECODER_H_
#define __DECODER_H_

#ifdef HAVE_CONFIG_H
#include "config.h"
#endif

#include <exception>
#include "rect.h"

class blitter;
class chunck;

/*!	@brief base class for codec decompression.

	This class provides an interface for codec decompression.

*/
class decoder {
private:
	rect _r;			//!< current size of the frame object to decode
	point _p;			//!< position of the frame object to decode
protected:
	const rect & get_rect() const throw() { return _r; }
	const point & get_size() const throw() { return _p; }
public:
	decoder() throw() {};
	virtual ~decoder() throw() {};
	virtual bool init_size(const point & p, const rect & r) throw(std::exception) { _p = p; _r = r; return true; };
	virtual bool decode(blitter &, chunck &) throw(std::exception) = 0;
};

#endif
