/*
Copyright (c) 2002 Xavier Trochu

This software is provided 'as-is', without any express or implied warranty. In no event
will the authors be held liable for any damages arising from the use of this software.

Permission is granted to anyone to use this software for any purpose, including commercial
applications, and to alter it and redistribute it freely, subject to the following
restrictions:

1. The origin of this software must not be misrepresented; you must not claim that you
   wrote the original software. If you use this software in a product, an acknowledgment
   in the product documentation would be appreciated but is not required.

2. Altered source versions must be plainly marked as such, and must not be misrepresented 
   as being the original software.

3. This notice may not be removed or altered from any source distribution.
*/
#include <sstream>
#include <stdexcept>
#include "rect.h"
#include <iostream>

using namespace std;

rect::rect() throw() : _topleft(0, 0), _bottomright(0,0) { 
}

rect::rect(int x, int y) throw(exception) : _topleft(0, 0), _bottomright(x, y) {
	check();
}

rect::rect(int x1, int y1, int x2, int y2) throw(exception) : _topleft(x1, y1), _bottomright(x2, y2) { 
	check(); 
}

rect::rect(const rect & r) throw() : _topleft(r._topleft), _bottomright(r._bottomright) {
}

rect::rect & rect::operator=(const rect & r) throw() {
	_topleft = r._topleft;
	_bottomright = r._bottomright;
	return *this;
}

bool rect::operator==(const rect & r) const throw() {
	return _topleft == r._topleft && _bottomright == r._bottomright;
}

void rect::check() throw(exception) { 
	if ((_topleft.get_x() < 0) || (_bottomright.get_x() < _topleft.get_x())  || (_topleft.get_y() < 0) || (_bottomright.get_y() < _topleft.get_y()))
	{
		ostringstream oss;
		oss << "Invalid rect : " << *this;
		throw logic_error(oss.str());
	}
}

//~ bool rect::is_inside(const rect & r) const throw() {
	//~ return _topleft.get_x() >= r._topleft.get_x() && _bottomright.get_x() <= r._bottomright.get_x() && _topleft.get_y() >= r._topleft.get_y() && _bottomright.get_y() <= r._bottomright.get_y();
//~ }

bool rect::is_inside(int x, int y) const throw() {
	return _topleft.get_x() >= x && _bottomright.get_x() < x && _topleft.get_y() >= y && _bottomright.get_y() < y;
}

bool rect::is_inside(const point & p) const throw() {
	return (left() <= p.get_x()) && (right() > p.get_x()) && (top() <= p.get_y()) && (bottom() > p.get_y());
}

//~ bool rect::clip(rect & r) const throw(exception) {
	//~ // clip r in this, return true if clip performed
	//~ // throw an exception if the rect is invalid after the clip
	//~ if(!r.is_inside(*this)) {
		//~ ostringstream oss;
		//~ oss << "Invalid clip of " << r << " by " << *this;
		//~ throw logic_error(oss.str());
	//~ }
	//~ bool changed = false;
	//~ if(_topleft.get_x() > r._topleft.get_x()) { r._topleft.get_x() = _topleft.get_x(); changed = true; }
	//~ if(_topleft.get_y() > r._topleft.get_y()) { r._topleft.get_y() = _topleft.get_y(); changed = true; }
	//~ if(_bottomright.get_x() < r._bottomright.get_x()) { r._bottomright.get_x() = _bottomright.get_x(); changed = true; }
	//~ if(_bottomright.get_y() < r._bottomright.get_y()) { r._bottomright.get_y() = _bottomright.get_y(); changed = true; }
	//~ return changed;
//~ }

ostream & operator<<(ostream & o, const rect & r) {
	o << "[" << r.left() << "-" << r.top() << ";" << r.right() << "-" << r.bottom() <<"]";
	return o;
}

ostream & operator<<(ostream & o, const point & p) {
	o << "(" << p.get_x() << "-" << p.get_y()  <<")";
	return o;
}


